/*
 * Copyright (c) 2007 Jan Hudecek
 * Copyright (c) 2005 Jakub Jermar
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <print.h>

#include <test.h>
#include <proc/tasklet.h>
#include <synch/waitq.h>

#include <arch/types.h>

static void func(void *data)
{
	printf("%s", data);
}

char * test_tasklet1(bool quiet)
{
	waitq_t wq;
	waitq_initialize(&wq);
	tasklet_descriptor_t *tasklet_desc;

	//before we start we need to register a tasklet
	if (!quiet)
		printf("Registering tasklet...");
	if (!quiet)
		tasklet_desc=tasklet_register(&func, "\nTasklet called and received data\n");
	else
		tasklet_desc=tasklet_register(&func, "");
	if (!quiet)
		printf("Done!\n");

	//first we'll try disabling the tasklet
	if (!quiet)
		printf("Disabling tasklet...");
	tasklet_disable(tasklet_desc);
	if (!quiet)
		printf("Done!\n");

	//we'll schedule the disabled tasklet
	if (!quiet)
		printf("Scheduling tasklet...");
	tasklet_schedule(tasklet_desc);
	if (!quiet)
		printf("Done!\n");

	//and we'll wait if it gets called. It shouldn't however because it's disabled
	if (!quiet)
		printf("Waiting 5s...");
	waitq_sleep_timeout(&wq,(uint32_t) 5000000,0);
	if (!quiet)
		printf("Done!\n");

	//then we'll try to enable it
	if (!quiet)
		printf("Enabling tasklet...");
	tasklet_enable(tasklet_desc);
	if (!quiet)
		printf("Done!\n");
	
	//and wait if it gets called this time. It should beacause it's enabled
	if (!quiet)
		printf("Waiting 5s...");
	waitq_sleep_timeout(&wq,(uint32_t) 5000000,0);
	if (!quiet)
		printf("Done!\n");

	//finally we'll free the tasklet structure
	if (!quiet)
		printf("Freeing...");
	tasklet_free(tasklet_desc);
	if (!quiet)
		printf("Done!\n");


	return NULL;
}
